import { createSignal } from "solid-js";

export function PersonRegister(props: {dni: string, onRegister: () => void}) {
    const [paternalSurname, setPaternalSurname] = createSignal("");
    const [maternalSurname, setMaternalSurname] = createSignal("");
    const [names, setNames] = createSignal("");
    const [loading, setLoading] = createSignal(false);

    const register = async() => {
        setLoading(true);

        const paternal = paternalSurname();
        const maternal = maternalSurname();
        const name = names();

        if (paternal === "" || maternal === "" || name === "") {
            return;
        }

        const data = {
            person_dni: props.dni,
            person_names: name.toUpperCase(),
            person_paternal_surname: paternal.toUpperCase(),
            person_maternal_surname: maternal.toUpperCase(),
        };

        const response = await fetch(`${import.meta.env.VITE_BACKEND_URL}/api/person`, {
            method: "POST",
            headers: {
                "Content-Type": "application/json",
            },
            body: JSON.stringify(data),
        });

        if (response.ok) {
            props.onRegister();
        }

        setLoading(false);
    };

    return (
        <div class="relative max-w-[14rem] mx-auto">
            <form
                onSubmit={(ev) => {
                    ev.preventDefault();
                    register();
                }}
            >
                <MaterialInput
                    name="Apellido paterno"
                    value={paternalSurname()}
                    setValue={(v) => setPaternalSurname(v.toUpperCase())}
                    loading={loading()}
                />
                <MaterialInput
                    name="Apellido materno"
                    value={maternalSurname()}
                    setValue={(v) => setMaternalSurname(v.toUpperCase())}
                    loading={loading()}
                />
                <MaterialInput
                    name="Nombres"
                    value={names()}
                    setValue={(v) => setNames(v.toUpperCase())}
                    loading={loading()}
                />

                <button
                    class="bg-c-primary text-c-on-primary px-4 py-2 rounded-full cursor-pointer
                                        disabled:opacity-50 disabled:cursor-not-allowed"
                >
                    Registrar
                </button>
            </form>
        </div>
    );
}

function MaterialInput(props: {
    name: string,
    loading: boolean,
    value: string,
    setValue: (v: string) => void,
}) {
    let inputElement: HTMLInputElement | undefined;

    return (
        <div class="relative my-6">
            <input
                ref={inputElement}
                id="search-dni"
                class="bg-c-background text-c-on-background border-c-outline border-2 rounded px-2 py-1 w-full
                        invalid:border-c-error invalid:text-c-error
                        focus:border-c-primary outline-none font-mono
                        disabled:opacity-50 disabled:cursor-not-allowed"
                type="text"
                placeholder={props.name}
                value={props.value}
                required
                onInput={(e) => props.setValue(e.target.value)}
                disabled={props.loading}
            />


            <label for="search-dni" class="absolute -top-2 left-2 text-xs bg-c-surface px-1 select-none">
                {props.name}
            </label>
        </div>
    );
}
